#include <windows.h>
#include <windowsx.h>

#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>
#include <math.h>
#include <stdio.h>

//#define _FULL_SCREEN		//         
#define MAX_STRING	1024
#define MAX_VERTS	20000

// Global variables
char			lpszAppName[]	= "2D Stencil";
char			lpszClassName[]	= "2D Stencil";
static	HGLRC	hRC;
static	HDC		hDC;
static	BOOL	m_bActive = FALSE;


// Functions
GLvoid Initial(GLsizei Width, GLsizei Height);
GLvoid Resize(GLsizei Width, GLsizei Height);
GLvoid CalcList(void);
GLvoid Draw(RECT* rect);
GLvoid StartRC(HWND hWnd);
GLvoid StopRC(HWND hWnd);
LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);


// Implementation
GLvoid DrawPrim()
{
	glStencilFunc(GL_ALWAYS, 2, 1);
	glStencilOp(GL_KEEP, GL_REPLACE, GL_KEEP);
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_POLYGON);
		glVertex3f(1.0f, 1.0f, 0.0f);
		glVertex3f(-1.0f, 1.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, 0.0f);
		glVertex3f(1.0f, -1.0f, 0.0f);
	glEnd();

	glStencilFunc(GL_NEVER, 1, 1);
	glStencilOp(GL_REPLACE, GL_KEEP, GL_KEEP);
	glColor3f(0.0f, 1.0f, 0.0f);
	glBegin(GL_POLYGON);
		glVertex3f(0.0f, 1.5f, 0.0f);
		glVertex3f(-1.5f, -1.5f, 0.0f);
		glVertex3f(1.5f, -1.5f, 0.0f);
	glEnd();

	glStencilFunc(GL_EQUAL, 2, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	glColor3f(0.0f, 0.0f, 1.0f);
	glBegin(GL_POLYGON);
		glVertex3f(0.0f, 2.0f, 0.0f);
		glVertex3f(-2.0f, -2.0f, 0.0f);
		glVertex3f(2.0f, -2.0f, 0.0f);
	glEnd();
}


GLvoid Initial(GLsizei w, GLsizei h)
{
	Resize (w, h); 
}


GLvoid StartRC(HWND hWnd)
{
	glClearColor (0.5, 0.5, 0.75, 1.0);
	glClearStencil(0);
	glStencilMask(1);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_STENCIL_TEST);
	glEnable(GL_COLOR_MATERIAL);
}

GLvoid StopRC(HWND hWnd)
{
	if(glIsEnabled(GL_LIGHTING))
		glDisable(GL_LIGHTING);
	if(glIsEnabled(GL_LIGHT0))
		glDisable(GL_LIGHT0);
	if(glIsEnabled(GL_COLOR_MATERIAL))
		glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_DEPTH_TEST);
	if(glIsEnabled(GL_STENCIL_TEST))
		glDisable(GL_STENCIL_TEST);
}

GLvoid Resize(GLsizei w, GLsizei h)
{
	if(w>=h)
		glViewport ((w-h)/2, 0, (GLsizei) h, (GLsizei) h); 
	if(w<h)
		glViewport (0, (h-w)/2, (GLsizei) w, (GLsizei) w); 

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	glFrustum (-1, 1, -1, 1, 4, 15);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();

 	glTranslatef(0.0, 0.0, -14.0);
}


GLvoid Draw()
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	glPushMatrix();

	DrawPrim();

	glPopMatrix();

	SwapBuffers(hDC);
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT Screen;
	GLuint PixelFormat;
	static GLfloat red, green, blue;
	static BOOL light = TRUE;
	static BOOL color = TRUE;

	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),		// 
		1,									//  ?
		PFD_DRAW_TO_WINDOW|					// format must support Window
		PFD_SUPPORT_OPENGL|					// format must support OpenGL
		PFD_DOUBLEBUFFER,					// must support double buffer
		PFD_TYPE_RGBA,						//  RGBA 
		16,									// 16Bit color depth
		0, 0, 0, 0, 0, 0,					// Color bits ignored ?
		0,									// No Alpha buffer
		0,									// shift bit ignored
		0,									// no accumulation buffer
		0, 0, 0, 0,							// accumulation buffer bits ignored
		16,									// 16bit z-buffer (depth buffer)
		8,									// no stencil buffer
		0,									// no auxiliary buffer
		PFD_MAIN_PLANE,						// main drawing layer
		0,									// reserved
		0, 0, 0								// layer mask ignored
	};

	GetClientRect(hWnd, &Screen);

	switch(msg)
	{
	case WM_CREATE:
		hDC = GetDC(hWnd);
		PixelFormat = ChoosePixelFormat(hDC, &pfd);

		if(!PixelFormat){
			MessageBox(0, "Can't find suitable PixelFormat","Error", MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		if(!SetPixelFormat(hDC, PixelFormat, &pfd)) {
			MessageBox(0, "Can't set The PixelFormat","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		hRC = wglCreateContext(hDC);
		if(!hRC) {
			MessageBox(0, "Can't Create Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		if(!wglMakeCurrent(hDC, hRC)) {
			MessageBox(0, "Can't set current Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		Initial(Screen.right, Screen.bottom);
		StartRC(hWnd);
		break;
	case WM_ACTIVATE:
		m_bActive = !((BOOL)HIWORD(wParam));
		break;
	case WM_CLOSE:
	case WM_DESTROY:
		StopRC(hWnd);
		ChangeDisplaySettings(NULL, 0);

		wglMakeCurrent(hDC, NULL);
		wglDeleteContext(hRC);
		ReleaseDC(hWnd, hDC);

		PostQuitMessage(0);
		break;
	case WM_KEYDOWN:
		if(wParam == VK_ESCAPE) SendMessage(hWnd, WM_CLOSE, 0, 0);
		break;
	case WM_KEYUP:
		break;
	case WM_SIZE:
		Resize(LOWORD(lParam), HIWORD(lParam));
		break;
	default:
		return( DefWindowProc(hWnd, msg, wParam, lParam));
	}
	return(0);
}


int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmd, int nShowCmd)
{
	MSG			msg;
	WNDCLASS	wc;
	HWND		hWnd;

	wc.style		= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc	= (WNDPROC)WndProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInst;
	wc.hIcon		= NULL;
	wc.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= lpszClassName;

	if(!RegisterClass(&wc)) 
	{
		MessageBox(0,"Error Create window class","Error", MB_OK|MB_ICONERROR);
		return FALSE;
	}

#ifdef _FULL_SCREEN
	hWnd = CreateWindow(lpszClassName,
		lpszAppName,
		WS_POPUP |
		WS_CLIPCHILDREN |
		WS_CLIPSIBLINGS,
		0, 
		0, 
		GetSystemMetrics(SM_CXSCREEN), 
		GetSystemMetrics(SM_CYSCREEN), 
		NULL, 
		NULL, 
		hInst, 
		NULL);

	if(!hWnd){
		MessageBox(0,"Error Create window","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}

	DEVMODE dmScreenSettings;

	memset(&dmScreenSettings, 0, sizeof(DEVMODE));
	dmScreenSettings.dmSize			= sizeof(DEVMODE);
	dmScreenSettings.dmPelsWidth	= GetSystemMetrics(SM_CXSCREEN);
	dmScreenSettings.dmPelsHeight	= GetSystemMetrics(SM_CYSCREEN);
	dmScreenSettings.dmFields		= DM_PELSWIDTH | DM_PELSHEIGHT;
	ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
#else
	hWnd = CreateWindow(lpszClassName,
		lpszAppName,
		WS_OVERLAPPEDWINDOW |
		WS_CLIPCHILDREN |
		WS_CLIPSIBLINGS,
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		NULL, 
		NULL, 
		hInst, 
		NULL);

	if(!hWnd){
		MessageBox(0,"Error Create window","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}
#endif

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

    while (TRUE)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                return msg.wParam;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else if (m_bActive)
        {
            Draw();
        }
        else
        {
            // Make sure we go to sleep if we have nothing else to do
            WaitMessage();
        }
    }
	return 0;
}

